/*
 * File shmalloc.h
 *
 * Authors Philip Duval (DESY), Steve Herb (DESY), Marina Nikolova (EMBL), 
 *    Karol Bartkiewicz (DESY), Marcus Walla (DESY), Jan Szczesny (DESY),
 *    Arthur Aghababyan (DESY), Stefan Weisse (DESY), Tobias Tempel (DESY)
 *
 * Copyright (c) Deutsches Elektronen-Synchrotron DESY 1992-2023
 *
 * The TINE .c codeset made available by open source license ultimately
 * stems from the original ISOLDE control system ((c)CERN, released under
 * a permissive CERN licence akin to the later well known MIT licence)
 * in use at CERN in 1992.  Although none of the current software is in
 * anyway a redistribution of software from the ISOLDE system of that
 * era (apart from various error/status code definitions), it does in
 * fact inherit several conceptual ideas.  We therefore owe a substantial
 * debt to CERN for its inception, and would be remiss if we did not express
 * our gratitude in the TINE open source headers.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef _SHMALLOC_H_
#define _SHMALLOC_H_
#include "prolog.h"

#define MMF_MSGLOOP

#define hSHARE INVALID_HANDLE_VALUE

#define WM_REQ_LCL        (WM_USER+100)
#define WM_RCV_LCL        (WM_USER+101)
#define WM_STOP_LCL       (WM_USER+102)

#define WORK_BUFFER_SIZE 0x10000 //0x2000
#define SHM_BUFFER_SIZE WORK_BUFFER_SIZE
extern UINT32 gShmBufferSize;

typedef struct 
{
  char dataTag[16];
  int  dirty;
  int  size;
} shmDataGram;

#define shmPayload(d) ((BYTE *)(d) + sizeof(shmDataGram))

extern HWND gInLclAdr;
extern HWND hShmWnd;
extern HINSTANCE hParent;
extern int gForceShmDatagrams;

HANDLE mmfalloc(char *tag,int SizeInBytes,BYTE **lpView,int *mmfFlag);
int mmfInit(void);
shmDataGram *shmGetDataGram(char *tag,int size,HANDLE *h);
int shmSndDataGram(ClnHdr *cln,char *tag,BYTE *data,int size,HANDLE *h,UINT iMessage);
void shmFreeDataGram(shmDataGram *dg,HANDLE h);
BYTE *shmDataBlockAlloc(char *tag,int size,HANDLE *hBlock);
void shmDataBlockFree(BYTE *d,HANDLE hBlock);
int CreateShmWindow(void);
int shmStart(void);
int shmStop(void);
void DestroyShmWindow(void);
TINE_EXPORT void SetForceShmDatagrams(int value);
int sendShmData(ClnHdr *cln,BYTE *data,int size,UINT iMessage);
void DoEvents(int maxevents);
void DoClientEvents(int maxevents);
void DoServerEvents(int maxevents);
TINE_EXPORT void SetMMFMessageSize(int sizeInBytes);

#endif /* _SHMALLOC_H_ */
